from NL.CommonConcepts.Configurations.MoleculeConfiguration import MoleculeConfiguration
from NL.CommonConcepts.PhysicalQuantity import Angstrom

from API import ExportConfigurationPlugin, showMessage

class XYZExporterPlugin(ExportConfigurationPlugin):
    """ Class for handling the export of the XYZ input files. """

    def title(self):
        """ Return the title file selection dialog. """

        return 'XYZ'

    def extension(self):
        """ The default extension of XYZ. """

        return 'xyz'

    def export(self, configuration, path):
        """
        Export the configuration.

        @param configuration : The configuration to export.
        @param path          : The path to save the configuration to.

        @return None
        """

        # XYZ files only supports molecules.
        if not isinstance(configuration, MoleculeConfiguration):
            showMessage('XYZExporter can only export MoleculeConfigurations')
            return

        # Open the file with write permission.
        with open(path, 'w') as f:

            # Get the total number of atoms.
            number_of_atoms = len(configuration)

            # Write out the header to the file.
            f.write('%i\n' % number_of_atoms)
            f.write('Generated by XYZExporter\n')

            # Get the list of atomic symbols.
            symbols = [ element.symbol() for element in configuration.elements() ]
            # Get the cartesian coordinates in units of Angstrom.
            coordinates = configuration.cartesianCoordinates().inUnitsOf(Angstrom)

            # Loop over each atom and write out its symbol and coordinates.
            for i in xrange(number_of_atoms):
                x, y, z = coordinates[i]
                f.write('%3s %16.8f %16.8f %16.8f\n' % (symbols[i], x, y, z))


    def canExport(self, configuration):
        """
        Method to determine if an exporter class can export a given configuration.

        @param configuration : The configuration to test.

        @return A bool, True if the plugin can export, False if it cannot.
        """

        supported_configurations = [MoleculeConfiguration]

        return isinstance(configuration, supported_configurations)
